package com.zjyr.beidouservice.adapter.impl.netty;

import com.zjyr.beidouservice.adapter.impl.netty.decode.YuanRongProtocolDecode;
import com.zjyr.beidouservice.adapter.impl.netty.encode.ProtocolStartEncode;
import com.zjyr.beidouservice.adapter.impl.netty.encode.YuanRongProtocolEncode;
import com.zjyr.beidouservice.config.NettySocketConfigure;
import com.zjyr.beidouservice.pojo.vo.binary.BaseBinaryProtocol;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.socket.SocketChannel;
import io.netty.handler.codec.LengthFieldBasedFrameDecoder;
import io.netty.handler.timeout.IdleStateHandler;
import jakarta.annotation.Resource;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Component;

import java.util.concurrent.TimeUnit;

/**
 * The type Channel init.
 *
 * @param <T> the type parameter
 */
@EqualsAndHashCode(callSuper = true)
@Component
@RequiredArgsConstructor
@Data
public class ChannelInit<T> extends ChannelInitializer<SocketChannel> {
    /**
     * The constant MAX_FRAME_LENGTH.
     */
    private static final int MAX_FRAME_LENGTH       = 1024 * BaseBinaryProtocol.MAX_LEN;  //最大长度
    /**
     * The constant LENGTH_FIELD_LENGTH.
     */
    private static final int LENGTH_FIELD_LENGTH    = 2;  //长度字段所占的字节数
    /**
     * The constant LENGTH_FIELD_OFFSET.
     */
    private static final int LENGTH_FIELD_OFFSET    = BaseBinaryProtocol.START.length();  //长度偏移
    /**
     * The constant LENGTH_ADJUSTMENT.
     */
    private static final int LENGTH_ADJUSTMENT      = 0;
    /**
     * The constant INITIAL_BYTES_TO_STRIP.
     */
    private static final int INITIAL_BYTES_TO_STRIP = BaseBinaryProtocol.START.length();

    /**
     * The Message handler.
     */
    @Resource
    private MessageHandler<T> messageHandler;

    @Resource
    private NettySocketConfigure nettySocketConfigure;

    @Override
    protected void initChannel(SocketChannel channel) {
        channel.pipeline()
               .addLast(new IdleStateHandler(0, 0, nettySocketConfigure.getHeartTimeout(), TimeUnit.SECONDS))
               .addLast(new LengthFieldBasedFrameDecoder(MAX_FRAME_LENGTH,
                                                         LENGTH_FIELD_OFFSET,
                                                         LENGTH_FIELD_LENGTH,
                                                         LENGTH_ADJUSTMENT,
                                                         INITIAL_BYTES_TO_STRIP))
               .addLast(new YuanRongProtocolDecode())
               .addLast(new ProtocolStartEncode(2))
               .addLast(new YuanRongProtocolEncode<T>())
               .addLast("message", messageHandler);
    }
}
