package com.zjyr.beidouservice.controller;

import com.zjyr.beidouservice.pojo.MyResp;
import com.zjyr.beidouservice.pojo.dto.AlarmDevRespDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceReqDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceRespDTO;
import com.zjyr.beidouservice.pojo.dto.ManufacturerRespDTO;
import com.zjyr.beidouservice.service.DeviceService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * 终端设备
 */
@RestController
@RequestMapping("/")
@Validated
@Slf4j
public class DeviceController {

    @Resource
    DeviceService deviceService;

    /**
     * 获取省份信息
     *
     * @return 省份信息
     */
    @PostMapping("device/location")
    public MyResp<List<DeviceRespDTO>> getDeviceInfo(@RequestBody DeviceReqDTO deviceReqDTO) {
        List<DeviceRespDTO> deviceRespDTO = deviceService.getDeviceInfo(deviceReqDTO);
        return MyResp.success(deviceRespDTO);
    }

    /**
     * 获取厂家信息
     *
     * @return 厂家信息
     */
    @PostMapping("device/manufacturer")
    public MyResp<ManufacturerRespDTO> getDeviceManufacturerInfo() {
        ManufacturerRespDTO manufacturerRes = deviceService.getDeviceManufacturerInfo();
        return MyResp.success(manufacturerRes);

    }

    /**
     * 获取警报终端信息
     *
     * @return 省份信息
     */
    @PostMapping("alarm/device")
    public MyResp<AlarmDevRespDTO> getAlarmDeviceInfo() {
        AlarmDevRespDTO alarmDevRespDTO = deviceService.getAlarmDeviceInfo();
        return MyResp.success(alarmDevRespDTO);

    }

}
