package com.zjyr.beidouservice.controller;

import com.zjyr.beidouservice.pojo.MyResp;
import com.zjyr.beidouservice.pojo.dto.city.CityRespDTO;
import com.zjyr.beidouservice.pojo.dto.city.CitySearchDTO;
import com.zjyr.beidouservice.pojo.dto.province.OnlyProvinceResp;
import com.zjyr.beidouservice.pojo.dto.province.ProvinceSearchDTO;
import com.zjyr.beidouservice.pojo.dto.province.ProvinceRespDTO;
import com.zjyr.beidouservice.service.LocationService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 省份、地市、区描述
 */
@RestController
@RequestMapping("/")
@Validated
@Slf4j
public class LocationController {

    @Resource
    LocationService locationService;


    /**
     * 获取省份信息
     *
     * @return 省份信息
     */
    @PostMapping("only/provinces")
    public MyResp<OnlyProvinceResp> getOnlyProvinces() {
        OnlyProvinceResp onlyProvinceResp = locationService.getOnlyProvinces();
        return MyResp.success(onlyProvinceResp);
    }
    /**
     * 获取省份及省份对应的地市信息
     *
     * @return 省份及省份对应的地市信息
     */
    @PostMapping("provinces")
    public MyResp<ProvinceRespDTO> getProvinces(@RequestBody ProvinceSearchDTO provinceSearchDTO) {
        ProvinceRespDTO provinceResp = locationService.getProvinces(provinceSearchDTO);
        return MyResp.success(provinceResp);
    }


    /**
     * 获取地市及地市对应的区信息
     *
     * @return 地市及地市对应的区信息
     */
    @PostMapping("citys")
    public MyResp<CityRespDTO> getCity(@RequestBody CitySearchDTO citySearchDTO) {
        CityRespDTO cityResp = locationService.getCity(citySearchDTO);
        return MyResp.success(cityResp);
    }


}
