package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.common.impl.SensorControlModeName;
import com.zjyr.beidouservice.common.impl.SensorControlTunnelName;
import com.zjyr.beidouservice.pojo.po.BeidouAdapterControlContent;
import com.zjyr.beidouservice.pojo.vo.binary.BaseBinaryProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.HeartProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.MessageContent;
import com.zjyr.beidouservice.pojo.vo.binary.QuerySensorProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.SensorControlProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.SensorControllContent;
import com.zjyr.beidouservice.service.AdapterProtocolService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class BeidouSocketProtocolServiceImpl implements AdapterProtocolService {
    @Override
    public HeartProtocol createHeartProtocol() {
        return new HeartProtocol();
    }

    @Override
    public List<SensorControllContent> createSensorCtrlInfo(List<BeidouAdapterControlContent> ctxList) {
        List<SensorControllContent> list = new ArrayList<>();

        for (BeidouAdapterControlContent v : ctxList) {
            list.add(SensorControllContent.builder()
                                          .districtsCode(v.getDistrictsCode().byteValue())
                                          //.controlAction(v.getControlAction())
                                          .sensorId(v.getSensorId().shortValue())
                                          .build());
        }

        return list;
    }

    @Override
    public SensorControlProtocol createSensorControlProtocol(SensorControlTunnelName tunnelName,
                                                             byte[] tunnelAddr,
                                                             Integer month,
                                                             Integer days,
                                                             Integer hour,
                                                             Integer minute,
                                                             Integer province,
                                                             Integer city,
                                                             SensorControlModeName ctrlMode,
                                                             Integer ctrlCmd,
                                                             List<SensorControllContent> ctrlItems) {
        return SensorControlProtocol.builder()
                                    .controlTunnel(tunnelName)
                                    .tunnelAddr(tunnelAddr)
                                    .month(month.byteValue())
                                    .days(days.byteValue())
                                    .hour(hour.byteValue())
                                    .minute(minute.byteValue())
                                    .province(province.byteValue())
                                    .city(city.byteValue())
                                    .controlMode(ctrlMode)
                                    .controlCommand(ctrlCmd.byteValue())
                                    .nControlInfo(((Integer) ctrlItems.size()).byteValue())
                                    .controllContents(new ArrayList<>(ctrlItems))
                                    .build();
    }

    @Override
    public QuerySensorProtocol createSensorQueryProtocol(Integer province,
                                                         Integer city,
                                                         List<SensorControllContent> ctrlItems) {
        return QuerySensorProtocol.builder()
                                  .province(province.byteValue())
                                  .city(city.byteValue())
                                  .nControlInfo(((Integer) ctrlItems.size()).byteValue())
                                  .controllContents(new ArrayList<>(ctrlItems))
                                  .build();
    }

    @Override
    public <T> Object createTransmissionProtocol(T proContent) {
        MessageContent<T> msgCont = MessageContent.<T>builder().msgBody(proContent).build();
        return BaseBinaryProtocol.<T>builder().msgContent(msgCont).build();
    }
}
